using System;
using System.ComponentModel;
using System.Xml.Serialization;

using Microsoft.Xna.Framework.Graphics;

namespace EQIA.EQIDL
{
    [TypeConverter(typeof(RGBConverter))]
    public class RGB
    {
        /// <summary>
        /// A standard white color to signify no tinting of the source texture.
        /// </summary>
        [XmlIgnore(), Browsable(false)]
        public static RGB White = new RGB(255, 255, 255, 255);

        private byte myRed;
        private byte myGreen;
        private byte myBlue;
        private byte myAlpha;

        /// <summary>
        /// Gets or Sets the red value.
        /// </summary>
        [XmlElement("R")]
        [Browsable(true), Description("Red value, 0 - 255")]
        [NotifyParentProperty(true)]
        public byte Red { get { return myRed; } set { myRed = (byte)Math.Max(Math.Min((int)value, 255), 0); } }

        /// <summary>
        /// Gets or Sets the green value.
        /// </summary>
        [XmlElement("G")]
        [Browsable(true), Description("Green value, 0 - 255")]
        [NotifyParentProperty(true)]
        public byte Green { get { return myGreen; } set { myGreen = (byte)Math.Max(Math.Min((int)value, 255), 0); } }

        /// <summary>
        /// Gets or Sets the blue value.
        /// </summary>
        [XmlElement("B")]
        [Browsable(true), Description("Blue value, 0 - 255")]
        [NotifyParentProperty(true)]
        public byte Blue { get { return myBlue; } set { myBlue = (byte)Math.Max(Math.Min((int)value, 255), 0); } }

        /// <summary>
        /// Gets or Sets the transparency value.
        /// </summary>
        [XmlElement("Alpha")]
        [Browsable(true), Description("Transparency value, 0 - 255")]
        [NotifyParentProperty(true)]
        public byte Alpha { get { return myAlpha; } set { myAlpha = (byte)Math.Max(Math.Min((int)value, 255), 0); } }

        /// <summary>
        /// Create a new color.
        /// </summary>
        public RGB()
            : this(0, 0, 0, 255)
        {
        }

        /// <summary>
        /// Create a new color.
        /// </summary>
        /// <param name="r">Red value.</param>
        /// <param name="g">Green value.</param>
        /// <param name="b">Blue value.</param>
        /// <param name="a">Transparency value.</param>
        public RGB(byte r, byte g, byte b, byte a)
        {
            myRed = r;
            myGreen = g;
            myBlue = b;
            myAlpha = a;
        }

        public override string ToString()
        {
            return myRed + ", " + myGreen + ", " + myBlue + ", " + myAlpha;
        }

        // TODO : Fix the problems that arise when implimenting these, really it makes no sense considering
        //  this is the way I was doing things for the past year, but now the xml serializer is having issues
        //  reflecting the Color class.

        //public static implicit operator Color(RGB color)
        //{
        //    return new Color(color.Red, color.Green, color.Blue, color.Alpha);
        //}

        //public static implicit operator RGB(Color color)
        //{
        //    return new RGB(color.R, color.G, color.B, color.A);
        //}

        //public static bool operator !=(RGB rgb1, RGB rgb2)
        //{
        //    return rgb1.GetHashCode() != rgb2.GetHashCode();
        //}

        //public static bool operator ==(RGB rgb1, RGB rgb2)
        //{
        //    return rgb1.GetHashCode() == rgb2.GetHashCode();
        //}

        //public override bool Equals(object obj)
        //{
        //    return this == (RGB)obj;
        //}

        //public override int GetHashCode()
        //{
        //    return myRed.GetHashCode() | myGreen.GetHashCode() | myBlue.GetHashCode() | myAlpha.GetHashCode();
        //}
    }

    /// <summary>
    /// Type converter that will convert a EQIA.EQIDL.RGB to string and string to EQIA.EQIDL.RGB.
    /// </summary>
    public class RGBConverter : ExpandableObjectConverter
    {
        public override bool CanConvertTo(ITypeDescriptorContext context, System.Type destinationType)
        {
            return destinationType == typeof(RGB) ?
                true :
                base.CanConvertTo(context, destinationType);
        }

        public override bool CanConvertFrom(ITypeDescriptorContext context, System.Type sourceType)
        {
            return (sourceType == typeof(string)) ?
                true :
                base.CanConvertFrom(context, sourceType);
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, System.Type destinationType)
        {
            if (value != null && (value.GetType() != typeof(RGB)))
                throw new Exception("'" + value.GetType() + "' can not be converted to type '" + typeof(RGB) + "'.");

            return (destinationType == typeof(string)) ?
                ((RGB)value).ToString() :
                base.ConvertTo(context, culture, value, destinationType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            if (value.GetType() == typeof(string))
            {
                try
                {
                    string[] args = value.ToString().Split(',');
                    return new RGB(byte.Parse(args[0]), byte.Parse(args[1]), byte.Parse(args[2]),byte.Parse(args[3]));
                }
                catch
                {
                    throw new Exception("Can not convert '" + value.ToString() + "' to type '" + typeof(RGB) + "'.");
                }
            }

            return base.ConvertFrom(context, culture, value);
        }
    }
}
